%%% Monte Carlo Dynamic Classifier tool - MCDC tool
% http://www.ismvideo.org/NDE0UP/MCDCtool/en/index.html

% Copyright (C) 2014-2015 Gareth W. Peters, Tomoko Matsui, Francois Septier, and Akira Tamamori

% This program is free software; you can redistribute it and/or modify it under the terms
% of the GNU General Public License as published by the Free Software Foundation; either
% version 2 of the License, or (at your option) any later version.

% This program is distributed in the hope that it will be useful, but WITHOUT ANY
% WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
% PARTICULAR PURPOSE.  See the GNU General Public License for more details.

% You should have received a copy of the GNU General Public License along with this
% program; if not, see <http://www.gnu.org/licenses/>.

%
% LaplaceDistribution.m
%
classdef LaplaceDistribution < Distribution

  properties (SetAccess = private)
    Location
    Scale
  end

  properties (GetAccess = protected, SetAccess = private)
    dim
  end

  methods (Access = public)

    function this = LaplaceDistribution(location, scale, dim)
      if nargin < 3
        dim = 1;
      end
      this.Location = location;
      this.Scale = scale;
      this.dim = dim;
    end

    function result = Pdf(this, x)
      % x : data sequence of size N * dim
      result = prod(exp(-abs(x - this.Location) / this.Scale) / (2 * this.Scale), 2);
    end

    function result = LogPdf(this, x)
      result = -sum(log(2 * this.Scale) + abs(x - this.Location) / this.Scale, 2);
    end

    function result = Random(this, n)
      if nargin < 2
        n = 1;
      end
      u = rand(n, this.dim) - 0.5;
      result = this.Location - this.Scale * sign(u) .* log(1 - 2 * abs(u));
    end

  end

end
