%%% Monte Carlo Dynamic Classifier tool - MCDC tool
% http://www.ismvideo.org/NDE0UP/MCDCtool/en/index.html

% Copyright (C) 2014-2015 Gareth W. Peters, Tomoko Matsui, Francois Septier, and Akira Tamamori

% This program is free software; you can redistribute it and/or modify it under the terms
% of the GNU General Public License as published by the Free Software Foundation; either
% version 2 of the License, or (at your option) any later version.

% This program is distributed in the hope that it will be useful, but WITHOUT ANY
% WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
% PARTICULAR PURPOSE.  See the GNU General Public License for more details.

% You should have received a copy of the GNU General Public License along with this
% program; if not, see <http://www.gnu.org/licenses/>.

%
% MCDCTest.m
%

function [result, FnState, FnObs] = MCDCTest(u, y, N, varargin)

  mcdcMatFile = varargin{1};

  [algorithm, in, out] = MCDCMatFile.Load(mcdcMatFile);

  gridData = GridData(in.xGrids);

  n = size(out.loglik, 1);
  for i = 1:n
    j = find(out.IDX == i);
    f{i} = SplineFun(in.stateSplineHandle, gridData, out.FVAt(min(j)));
    g{i} = SplineFun(in.obsSplineHandle, gridData, out.GVAt(min(j)));
    w(i) = size(j, 1);
  end

  FnState = @(x) AverageFun(f, w, x);
  FnObs   = @(x) AverageFun(g, w, x);

  q = 1;
  r = 0.1;

  particleFilter = ParticleFilter(in.x0, in.xaux', u', y', q, r, N, in.K);
  result = particleFilter.LogLikelihood(FnState, FnObs);

end

function result = SplineFun(handle, gridData, eqnValues)
  szEqnValues = size(eqnValues);
  eqnValues = reshape(eqnValues, szEqnValues(2:end));
  result = VectorValuedFunction(ModelFunctions2(handle, gridData, eqnValues));
end

function result = AverageFun(handles, weights, x)
  n = size(handles, 2);
  result = handles{1}(x) .* weights(1);
  for i = 2:n
    result = result + handles{i}(x) .* weights(i);
  end
  result = result ./ sum(weights);
end
