%%% Monte Carlo Dynamic Classifier tool - MCDC tool
% http://www.ismvideo.org/NDE0UP/MCDCtool/en/index.html

% Copyright (C) 2014-2015 Gareth W. Peters, Tomoko Matsui, Francois Septier, and Akira Tamamori

% This program is free software; you can redistribute it and/or modify it under the terms
% of the GNU General Public License as published by the Free Software Foundation; either
% version 2 of the License, or (at your option) any later version.

% This program is distributed in the hope that it will be useful, but WITHOUT ANY
% WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
% PARTICULAR PURPOSE.  See the GNU General Public License for more details.

% You should have received a copy of the GNU General Public License along with this
% program; if not, see <http://www.gnu.org/licenses/>.

%
% ModelKind.m
%
classdef ModelKind < handle

  enumeration
    State
    Observation
  end

  methods (Access = public)

    function meanFunction = GetMeanFunction(this, mcdcStrategy)
      switch (this)
      case ModelKind.State
        meanFunction = @mcdcStrategy.StateMean;
      case ModelKind.Observation
        meanFunction = @mcdcStrategy.ObsMean;
      end
    end

    function covarianceFunction = GetCovarianceFunction(this, mcdcStrategy)
      switch (this)
      case ModelKind.State
        covarianceFunction = @mcdcStrategy.StateSigma;
      case ModelKind.Observation
        covarianceFunction = @mcdcStrategy.ObsSigma;
      end
    end

    function kernelParameters = GetKernelParameters(this, mcdcOutput, iteration)
      switch (this)
      case ModelKind.State
        kernelParameters = mcdcOutput.SKPAt(iteration);
      case ModelKind.Observation
        kernelParameters = mcdcOutput.OKPAt(iteration);
      end
    end

    function kernelGenerators = GetKernelGenerators(this, mcdcInput)
      switch (this)
      case ModelKind.State
        kernelGenerators = mcdcInput.stateKernelGens;
      case ModelKind.Observation
        kernelGenerators = mcdcInput.obsKernelGens;
      end
    end

    function splineHandle = GetSplineHandle(this, mcdcInput)
      switch (this)
      case ModelKind.State
        splineHandle = mcdcInput.stateSplineHandle;
      case ModelKind.Observation
        splineHandle = mcdcInput.obsSplineHandle;
      end
    end

    function AddKernelParameters(this, mcdcOutput, kernelParameters)
      switch (this)
      case ModelKind.State
        mcdcOutput.AddSKP(kernelParameters);
      case ModelKind.Observation
        mcdcOutput.AddOKP(kernelParameters);
      end
    end

    function AddEquationValues(this, mcdcOutput, equationValues)
      switch (this)
      case ModelKind.State
        mcdcOutput.AddFV(equationValues);
      case ModelKind.Observation
        mcdcOutput.AddGV(equationValues);
      end
    end

  end

end
