%%% Monte Carlo Dynamic Classifier tool - MCDC tool
% http://www.ismvideo.org/NDE0UP/MCDCtool/en/index.html

% Copyright (C) 2014-2015 Gareth W. Peters, Tomoko Matsui, Francois Septier, and Akira Tamamori

% This program is free software; you can redistribute it and/or modify it under the terms
% of the GNU General Public License as published by the Free Software Foundation; either
% version 2 of the License, or (at your option) any later version.

% This program is distributed in the hope that it will be useful, but WITHOUT ANY
% WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
% PARTICULAR PURPOSE.  See the GNU General Public License for more details.

% You should have received a copy of the GNU General Public License along with this
% program; if not, see <http://www.gnu.org/licenses/>.

%
% PlotGraph.m
%
%
classdef PlotGraph < handle

  properties (Access = private)
    fig
    legendNames
  end

  methods (Access = public)

    function this = PlotGraph()
      this.fig = figure();
      set(this.fig, 'Visible', 'off');
      set(this.fig, 'PaperType', 'A4');
      set(this.fig, 'PaperOrientation', 'landscape');
      set(this.fig, 'PaperUnits', 'normalized');
      set(this.fig, 'PaperPosition', [0 0 1 1]);

      this.legendNames = {};
    end

    %
    % set title of graph
    %   input : same as title()
    %
    function Title(this, varargin)
      figure(this.fig);
      title(varargin{:});
    end

    %
    % set label of x-axis of graph
    %   input : same as xlabel()
    %
    function XLabel(this, varargin)
      figure(this.fig);
      xlabel(varargin{:});
    end

    %
    % set label of y-axis of graph
    %   input : same as ylabel()
    %
    function YLabel(this, varargin)
      figure(this.fig);
      ylabel(varargin{:});
    end

    %
    % display legend of graph
    %   legendName: Name of legend
    %   x: array of X values
    %   y: array of Y values
    %   varargin: same as extra argument of plot()
    %
    function AddSeries(this, legendName, x, y, varargin)
      figure(this.fig);
      hold all;
      if nargin >= 5
        plot(x, y, varargin{:});
      else
        plot(x, y);
      end
      this.legendNames = horzcat(this.legendNames, legendName);
    end

    %
    % save graph as PDF format
    %   filename: file name of PDF
    %
    function SaveAs(this, filename)
      figure(this.fig);
      legend(this.legendNames{:});
      print(this.fig, '-dpdf', filename);
    end

  end

  methods (Access = private)

    function delete(this)
      close(this.fig);
    end

  end

end
