%%% Monte Carlo Dynamic Classifier tool - MCDC tool
% http://www.ismvideo.org/NDE0UP/MCDCtool/en/index.html

% Copyright (C) 2014-2015 Gareth W. Peters, Tomoko Matsui, Francois Septier, and Akira Tamamori

% This program is free software; you can redistribute it and/or modify it under the terms
% of the GNU General Public License as published by the Free Software Foundation; either
% version 2 of the License, or (at your option) any later version.

% This program is distributed in the hope that it will be useful, but WITHOUT ANY
% WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
% PARTICULAR PURPOSE.  See the GNU General Public License for more details.

% You should have received a copy of the GNU General Public License along with this
% program; if not, see <http://www.gnu.org/licenses/>.

%
% RBFKernelGeneratorStrategyChoice2.m
%

classdef RBFKernelGeneratorStrategyChoice2 < RBFKernelGeneratorStrategy

  properties (Access = public)
    svar  % the variance of the random walk for the parameter sigma
    lvar  % the variance of the random walk for the parameter l
  end

  methods (Access = public)

    function this = RBFKernelGeneratorStrategyChoice2(algorithm, kind)
      this = this@RBFKernelGeneratorStrategy(algorithm, kind);
      this.svar = 1;
      this.lvar = 1;
    end

    function result = GetGenerator(this, j, in, out)
      if j == 1
        result = this.initial;
      else
        dim = size(this.initial, 2);
        for i = 1:dim
          prevParams = this.kind.GetKernelParameters(out, j - 1);
          sDist = this.GetDistribution(prevParams(1, i, 1), this.svar, this.initial{i}.SDistribution);
          lDist = this.GetDistribution(prevParams(1, i, 2), this.lvar, this.initial{i}.LDistribution);
          result{i} = RBFKernelGenerator(sDist, lDist);
        end
      end
    end

  end

  methods (Access = private)

    function result = GetDistribution(this, mu, variance, uniformDist)
      result = BoundedNormalDistribution(mu, variance, uniformDist.LowerBound, uniformDist.UpperBound);
    end

  end

end
