%%% Monte Carlo Dynamic Classifier tool - MCDC tool
% http://www.ismvideo.org/NDE0UP/MCDCtool/en/index.html

% Copyright (C) 2014-2015 Gareth W. Peters, Tomoko Matsui, Francois Septier, and Akira Tamamori

% This program is free software; you can redistribute it and/or modify it under the terms
% of the GNU General Public License as published by the Free Software Foundation; either
% version 2 of the License, or (at your option) any later version.

% This program is distributed in the hope that it will be useful, but WITHOUT ANY
% WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
% PARTICULAR PURPOSE.  See the GNU General Public License for more details.

% You should have received a copy of the GNU General Public License along with this
% program; if not, see <http://www.gnu.org/licenses/>.

%
% SkewTDistribution.m
%
classdef SkewTDistribution < Distribution

  properties (SetAccess = private)
    Nu
    Mu
    Sigma
    Gamma
  end

  methods (Access = public)

    function this = SkewTDistribution(nu, mu, sigma, gamma)
      this.Nu = nu;
      this.Mu = mu;
      this.Sigma = sigma;
      this.Gamma = gamma;
    end

    function result = Pdf(this, x)
      nu = this.Nu;
      gx = bsxfun(@times, this.Gamma, x);
      cdfval = tcdf(gx .* sqrt((1 + nu) ./ (x .^ 2 + nu)), nu + 1);
      pdfval = tpdf(x, nu);
      result = prod(2 * cdfval .* pdfval, 2);
    end

    function result = Random(this, n)
      W = 1 ./ gamrnd(this.Nu ./ 2, 2 ./ this.Nu, n, 1);
      X1 = bsxfun(@times, this.Gamma, W);
      X2 = bsxfun(@times, sqrt(W), mvnrnd(zeros(size(this.Mu)), this.Sigma, n));
      result = this.Mu + X1 + X2;
    end

  end

end
