%%% Monte Carlo Dynamic Classifier tool - MCDC tool
% http://www.ismvideo.org/NDE0UP/MCDCtool/en/index.html

% Copyright (C) 2014-2015 Gareth W. Peters, Tomoko Matsui, Francois Septier, and Akira Tamamori

% This program is free software; you can redistribute it and/or modify it under the terms
% of the GNU General Public License as published by the Free Software Foundation; either
% version 2 of the License, or (at your option) any later version.

% This program is distributed in the hope that it will be useful, but WITHOUT ANY
% WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
% PARTICULAR PURPOSE.  See the GNU General Public License for more details.

% You should have received a copy of the GNU General Public License along with this
% program; if not, see <http://www.gnu.org/licenses/>.

%
% TDistribution.m
%
classdef TDistribution < Distribution

  properties (SetAccess = private)
    Nu
  end

  properties (GetAccess = protected, SetAccess = private)
    dim
  end

  methods (Access = public)

    function this = TDistribution(nu, dim)
      if nargin < 2
        dim = 1;
      end
      this.Nu = nu;
      this.dim = dim;
    end

    function result = Pdf(this, x)
      result = prod(tpdf(x, this.Nu), 2);
    end

    function result = LogPdf(this, x)
      result = sum( ...
          gammaln((this.Nu + 1) / 2) ...
          - log(this.Nu * pi) / 2 - gammaln(this.Nu / 2) ...
          - (this.Nu + 1) * log(1 + (x .^ 2) / this.Nu) / 2 ...
        , 2);
    end

    function result = Random(this, n)
      if nargin < 2
        n = 1;
      end
      result = trnd(this.Nu, n, this.dim);
    end

  end

end
