%%% Monte Carlo Dynamic Classifier tool - MCDC tool
% http://www.ismvideo.org/NDE0UP/MCDCtool/en/index.html

% Copyright (C) 2014-2015 Gareth W. Peters, Tomoko Matsui, Francois Septier, and Akira Tamamori

% This program is free software; you can redistribute it and/or modify it under the terms
% of the GNU General Public License as published by the Free Software Foundation; either
% version 2 of the License, or (at your option) any later version.

% This program is distributed in the hope that it will be useful, but WITHOUT ANY
% WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
% PARTICULAR PURPOSE.  See the GNU General Public License for more details.

% You should have received a copy of the GNU General Public License along with this
% program; if not, see <http://www.gnu.org/licenses/>.

%
% TLocationScaleDistribution.m
%
classdef TLocationScaleDistribution < ContinuousUnivariateDistribution

  properties (SetAccess = private)
    Mean
    Variance
    Nu
  end

  methods (Access = public)

    function this = TLocationScaleDistribution(mean, variance, nu, dim)
      if nargin < 4
        dim = 1;
      end
      this = this@ContinuousUnivariateDistribution( ...
          makedist('tLocationScale', ...
             'mu', mean, ...
             'sigma', sqrt(variance), ...
             'nu', nu), ...
          dim);
      this.Mean = mean;
      this.Variance = variance;
      this.Nu = nu;
    end

    function result = LogPdf(this, x)
      sigma = sqrt(this.Variance);
      xnorm = (x - this.Mean) / sigma;
      result = sum( ...
          gammaln((this.Nu + 1) / 2) ...
          - log(sigma * sqrt(this.Nu * pi)) - gammaln(this.Nu / 2) ...
          - ((this.Nu + 1) / 2) * log(1 + xnorm .^ 2 / this.Nu) ...
        , 2);
    end

  end

end
