%%% Monte Carlo Dynamic Classifier tool - MCDC tool
% http://www.ismvideo.org/NDE0UP/MCDCtool/en/index.html

% Copyright (C) 2014-2015 Gareth W. Peters, Tomoko Matsui, Francois Septier, and Akira Tamamori

% This program is free software; you can redistribute it and/or modify it under the terms
% of the GNU General Public License as published by the Free Software Foundation; either
% version 2 of the License, or (at your option) any later version.

% This program is distributed in the hope that it will be useful, but WITHOUT ANY
% WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
% PARTICULAR PURPOSE.  See the GNU General Public License for more details.

% You should have received a copy of the GNU General Public License along with this
% program; if not, see <http://www.gnu.org/licenses/>.

%
% WeibullDistribution.m
%
classdef WeibullDistribution < ContinuousUnivariateDistribution

  properties (SetAccess = private)
    A % Scale parameter
    B % Shape parameter
  end

  methods (Access = public)

    function this = WeibullDistribution(a, b, dim)
      if nargin < 3
        dim = 1;
      end
      this = this@ContinuousUnivariateDistribution( ...
          makedist('Weibull', 'a', a, 'b', b), ...
          dim);
      this.A = a;
      this.B = b;
    end

    function result = LogPdf(this, x)
      a = this.A;
      b = this.B;
      result = this.dim * log(b / a) + sum((b - 1) * log(x / a) - (x / a) .^ b, 2);
    end

  end

end
