%%% Monte Carlo Dynamic Classifier tool - MCDC tool
% http://www.ismvideo.org/NDE0UP/MCDCtool/en/index.html

% Copyright (C) 2014-2015 Gareth W. Peters, Tomoko Matsui, Francois Septier, and Akira Tamamori

% This program is free software; you can redistribute it and/or modify it under the terms
% of the GNU General Public License as published by the Free Software Foundation; either
% version 2 of the License, or (at your option) any later version.

% This program is distributed in the hope that it will be useful, but WITHOUT ANY
% WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
% PARTICULAR PURPOSE.  See the GNU General Public License for more details.

% You should have received a copy of the GNU General Public License along with this
% program; if not, see <http://www.gnu.org/licenses/>.

%
% Linear state space model
%
%
% k1   = 0.1;
% k2   = 0.1;
% mass = 1.0;
% q    = 0.1;
% r    = 0.1;
% dt   = 1.0;

function [x, y, F, H, Q, R] = ...
    LinearStateSpaceModel(T, k1, k2, mass, q, r, dt)

  F = [1 dt; (-k2 / mass * dt) (1 - k1 / mass * dt)];
  H = [1 0];

  Q = cov([(dt * dt) / 2; dt] * q);
  R = r * r;

  x1(1) = 1;
  x2(1) = 0;

  for t = 1:T

    % Latent variables
    if t ~= 1
      xn = F * [x1(t - 1); x2(t - 1)] + mvnrnd(0, Q);
      x1(t) = xn(1);
      x2(t) = xn(2);
    end

    % Observations
    y1(t) = H * [x1(t); x2(t)] + mvnrnd(0, R);

  end

  x = [x1' x2'];
  y = [y1'];

end
