%%% Monte Carlo Dynamic Classifier tool - MCDC tool
% http://www.ismvideo.org/NDE0UP/MCDCtool/en/index.html

% Copyright (C) 2014-2015 Gareth W. Peters, Tomoko Matsui, Francois Septier, and Akira Tamamori

% This program is free software; you can redistribute it and/or modify it under the terms
% of the GNU General Public License as published by the Free Software Foundation; either
% version 2 of the License, or (at your option) any later version.

% This program is distributed in the hope that it will be useful, but WITHOUT ANY
% WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
% PARTICULAR PURPOSE.  See the GNU General Public License for more details.

% You should have received a copy of the GNU General Public License along with this
% program; if not, see <http://www.gnu.org/licenses/>.

function plotKitagawa_loglike (log_file)
    fid = fopen(log_file);

    % count number of lines in log_file
    current_line = 1;
    tline = fgets(fid);
    while ischar(tline)
        tline = fgets(fid);
        if (numel(strfind(tline, 'log-likelihood')) ~= 0)
            current_line = current_line + 1;
        end
    end
    log_like_whole = zeros(current_line - 1, 1);

    % back to front
    frewind(fid);

    % count number of 'accepted'
    % get log-likelihood
    tline = fgets(fid);
    current_line = 1;
    num_accepted = 1;
    while ischar(tline)
        tline = fgets(fid);
        if (numel(strfind(tline, 'log-likelihood')) ~= 0)
            C = textscan(tline, '%s %s %s %s %s %f, %s\n');
            log_like_whole(current_line) = C{6};
            if (strcmp(C{7}, 'accepted'))
                num_accepted = num_accepted + 1;
            end
            current_line = current_line + 1;
        end
    end

    % back to front
    frewind(fid);

    % get log-likelihood when accepted
    log_like_accepted = zeros(num_accepted - 1, 1);
    position_accepted = zeros(num_accepted - 1, 1);
    index_accepted = 1;
    current_line = 1;
    tline = fgets(fid);
    while ischar(tline)
        tline = fgets(fid);
        if (numel(strfind(tline, 'log-likelihood')) ~= 0)
            C = textscan(tline, '%s %s %s %s %s %f, %s\n');
            if (strcmp(C{7}, 'accepted'))
                log_like_accepted(index_accepted) = C{6};
                position_accepted(index_accepted) = current_line;
                index_accepted = index_accepted + 1;
            end
            current_line = current_line + 1;
        end
    end
    fclose(fid);

    log_like_data = [log_like_whole; log_like_accepted];
    troupname = [char(ones(length(log_like_whole), 1) * 'Whole   '); ...
                 char(ones(length(log_like_accepted), 1) * 'Accepted')];

    % make figure
    figure(31);
    set(0,'defaultAxesFontSize',20);
    paper_pos = get(gcf, 'PaperPosition');
    set(gcf, 'PaperPosition', [paper_pos(1) paper_pos(2) 48 21]);
    position = get(gcf, 'Position');
    position(3) = 1000;
    position(4) = 400;
    set(gcf, 'Position', position);

    % plot log-likelihood
    pos1 = subplot(1, 2, 1);
    plot(log_like_whole);
    hold on;
    stairs(position_accepted, log_like_accepted, 'ro', 'LineWidth', 2);
    hold off;
    xlabel('MCMC iteration');
    ylabel('Log-likelihood');
    legend({'Whole', 'Accepted'}, 'Location', 'Best');
    xlim([1 current_line-1]);

    % plot box-plot
    pos2 = subplot(1, 2, 2);
    boxplot(log_like_data, troupname);
    set(gca, 'XTick', 1:2);
    set(findobj(gca, 'Type', 'text'), 'FontSize', 20);
    ylabel('Log-likelihood');
    ylim([-15*10^5 0]);

    pos1_atr = get(pos1, 'Position');
    pos2_atr = get(pos2, 'Position');
    % specify horizontal position, vertical position, width, and height
    set(pos1, 'Position', [pos1_atr(1)-0.05, pos1_atr(2) pos1_atr(3)+0.20, pos1_atr(4)]);
    set(pos2, 'Position', [pos2_atr(1)+0.10, pos1_atr(2) pos2_atr(3)-0.05, pos1_atr(4)]);

    file_name = sprintf('plots/KitagawaModelPMCMC2_loglike.png');
    saveas(gcf, file_name);

    file_name = sprintf('plots/KitagawaModelPMCMC2_loglike.eps');
    print('-r0','-depsc2', file_name);
end
