%%% Monte Carlo Dynamic Classifier tool - MCDC tool
% http://www.ismvideo.org/NDE0UP/MCDCtool/en/index.html

% Copyright (C) 2014 Gareth W. Peters, Tomoko Matsui, Francois Septier, and Akira Tamamori

% This program is free software; you can redistribute it and/or modify it under the terms
% of the GNU General Public License as published by the Free Software Foundation; either
% version 2 of the License, or (at your option) any later version.

% This program is distributed in the hope that it will be useful, but WITHOUT ANY
% WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
% PARTICULAR PURPOSE.  See the GNU General Public License for more details.

% You should have received a copy of the GNU General Public License along with this
% program; if not, see <http://www.gnu.org/licenses/>.

%
% Sigmoid_PMCMC_MultipleKnownSequences.m
%

%%% Initialization

addpath('../');
% rng('shuffle');

%%% Generate a sample model

% X = {};
% Y = {};
% V = {};
% W = {};
% for n = 1:10
%   [x, y, v, w] = Sigmoid().generate(30 + n * 5);
%   X{end+1} = x;
%   Y{end+1} = y;
%   V{end+1} = v;
%   W{end+1} = w;
%   clear x y v w;
% end
% clear n;
load('Sigmoid_10sequences.mat');

%%% Setup the estimator

grids = { ...
  [-0.2*pi:0.1*pi:2.2*pi] ...
};

stateKernelGens = { ...
  RBFKernelGenerator(UniformDistribution(0.01, 10), UniformDistribution(0.01, 10)) ...
};

stateMeanFuncs = { ...
  @(x) pi ...
};

obsKernelGens = { ...
  RBFKernelGenerator(UniformDistribution(0.01, 10), UniformDistribution(0.01, 10)) ...
};

obsMeanFuncs = { ...
  @(x) 0 ...
};

aspect = Aspect();
aspect.LogFileName = '../logs/Sigmoid_PMCMC_MultipleKnownSequences.log';
aspect.MatFileNamePrefix = '../logs/Sigmoid_PMCMC_MultipleKnownSequences';
aspect.SavesIntermediateMat = true;
aspect.IntermediateMatInterval = 1000;

J = 1000; % MCMC iteration count

disp('PMCMC Estimation using SPAPI');

stateSplineHandle = @SimpleSpline;
obsSplineHandle = @SimpleSpline;
algorithm = PMCMC();

stateModel = GaussianProcessModel(ModelKind.State, algorithm);
stateModel.kernelGeneratorStrategy.svar = 1.0;
stateModel.kernelGeneratorStrategy.lvar = 1.0;
algorithm.SetStateModel(stateModel);

obsModel = GaussianProcessModel(ModelKind.Observation, algorithm);
obsModel.kernelGeneratorStrategy.svar = 1.0;
obsModel.kernelGeneratorStrategy.lvar = 1.0;
algorithm.SetObsModel(obsModel);

x0 = 0;
xaux = [];
u = [];
q = 1;
r = 1;
N = 1000;

% Set likelihood calculator object if state sequence is also known.

% likelihoodCalculator = PMCMCParticleFilter(x0, xaux, u, Y, q, r, N, 0);
likelihoodCalculator = MultipleKnownSequences(X, xaux, u, Y, q, r, 0);
algorithm.SetLikelihoodCalculator(likelihoodCalculator);

%%% Run the estimator

[ IDX, SKP, OKP, FV, GV, XE, YE, loglik ] = MCDCTrain( ...
  algorithm, ...
  grids, ...
  stateKernelGens, ...
  obsKernelGens, ...
  stateMeanFuncs, ...
  obsMeanFuncs, ...
  0, ... % Not used in PMCMC estimation
  stateSplineHandle, ...
  obsSplineHandle, ...
  x0, ...
  [], ... % No auxiliary states
  u', ... % No controls
  Y', ...
  N, ...
  J, ...
  0, ...
  aspect ...
);

save('../logs/Sigmoid_PMCMC_MultipleKnownSequences.mat');

disp('Done');
